
# commandArgs()[1] = HTSeq count table
# commandArgs()[2] = prefix of output files

#----- library -----#
library(DESeq2)
#-------------------#


#----- input main arguments -----#
args <- commandArgs(trailingOnly = TRUE)
inputFile <- args[1]
outputPrefix <- args[2]
data <- read.table(inputFile, header = T, row.names = 1)
outputNormalizedCount <- paste(outputPrefix, ".normalized_count", sep = "")
outputNormalizedMean <- paste(outputPrefix, ".normalized_mean", sep = "")
outputTestResults <- paste(outputPrefix, ".results", sep = "")
output05SignResults <- paste(outputPrefix, ".05sign", sep = "")
#--------------------------------#


#----- input HTSeq count data -----#
colData = data.frame(condition = rep(c("NORM", "PAPI", "CARC", "META"), each = 2), type = "single-end", row.names = colnames(data))
dds <- DESeqDataSetFromMatrix(countData = data, colData = colData, design = ~ condition)
#----------------------------------#


#----- perform statistical test -----#
dds <- DESeq(dds, fitType = "local")
NP <- results(dds, contrast = c("condition", "PAPI", "NORM"))
NC <- results(dds, contrast = c("condition", "CARC", "NORM"))
NM <- results(dds, contrast = c("condition", "META", "NORM"))
PC <- results(dds, contrast = c("condition", "CARC", "PAPI"))
PM <- results(dds, contrast = c("condition", "META", "PAPI"))
CM <- results(dds, contrast = c("condition", "META", "CARC"))
#------------------------------------#


#----- calc q-values -----#
np <- data.frame(transform(NP, sample1 = "NORM", sample2 = "PAPI"), row.names = paste(rownames(NP), "_NP", sep = ""))
nc <- data.frame(transform(NC, sample1 = "NORM", sample2 = "CARC"), row.names = paste(rownames(NC), "_NC", sep = ""))
nm <- data.frame(transform(NM, sample1 = "NORM", sample2 = "META"), row.names = paste(rownames(NM), "_NM", sep = ""))
pc <- data.frame(transform(PC, sample1 = "PAPI", sample2 = "CARC"), row.names = paste(rownames(PC), "_PC", sep = ""))
pm <- data.frame(transform(PM, sample1 = "PAPI", sample2 = "META"), row.names = paste(rownames(PM), "_PM", sep = ""))
cm <- data.frame(transform(CM, sample1 = "CARC", sample2 = "META"), row.names = paste(rownames(CM), "_CM", sep = ""))
all <- rbind(np, nc, nm, pc, pm, cm)
q <- p.adjust(all$pval, method = "BH")
result <- transform(all, qval = q)
#-------------------------#


#----- calc mean expression levels for each condition -----#
normalizedCount <- counts(dds, normalized = TRUE)
normalizedMean <- data.frame(NORM = rep(0, times = nrow(normalizedCount)), PAPI = rep(0, times = nrow(normalizedCount)), CARC = rep(0, times = nrow(normalizedCount)), META = rep(0, times = nrow(normalizedCount)), row.names = rownames(normalizedCount))

for(i in 1 : nrow(normalizedCount)){
      for(j in 0 : 3){
      	    SUM = 0
	    for(k in 1 : 2){
	    	  SUM = SUM + normalizedCount[i, (j * 2 + k)]
	    }
	    normalizedMean[i, (j + 1)] <- SUM / 2
      }
}
#----------------------------------------------------------#


#----- output results -----#
out <- file(outputNormalizedCount, "w")
write.table(normalizedCount, out, col.names = F, row.names = T, append = F, quote = F, sep = "\t")
close(out)

out <- file(outputNormalizedMean, "w")
write.table(normalizedMean, out, col.names = F, row.names = T, append = F, quote = F, sep = "\t")
close(out)

out <- file(outputTestResults, "w")
write.table(result, out, col.names = T, row.names = T, quote = F, sep = "\t")
close(out)

out <- file(output05SignResults, "w")
write.table(subset(result, qval < 0.05), out, col.names = T, row.names = T, quote = F, sep = "\t")
close(out)
#--------------------------#
