
# commandArgs()[1] = HTSeq count table
# commandArgs()[2] = prefix of output files
# commandArgs()[3] = num. of replicates
# commandArgs()[4] = threshold

#----- library -----#
library(DESeq2)
#-------------------#


#----- input main arguments -----#
args <- commandArgs(trailingOnly = TRUE)
inputFile <- args[1]
outputPrefix <- args[2]
NUM <- as.numeric(args[3])
threshold <- as.numeric(args[4])
data <- read.table(inputFile, header = T, row.names = 1)
outputTestResults <- paste(outputPrefix, ".results", sep = "")
outputSignResults <- paste(outputPrefix, ".significant", sep = "")
#--------------------------------#


#----- input HTSeq count data -----#
colData = data.frame(condition = rep(c("A", "B", "C", "D"), each = NUM), type = "single-end", row.names = colnames(data))
dds <- DESeqDataSetFromMatrix(countData = data, colData = colData, design = ~ condition)
#----------------------------------#

#----- estimate dispersions -----#
dds <- estimateSizeFactors(dds)
dds <- estimateDispersions(dds)
#--------------------------------#


#----- perform statistical test -----#
dds <- nbinomLRT(dds, full = ~ condition, reduced = ~ 1)
result <- results(dds)
#------------------------------------#

#----- output results -----#
out <- file(outputTestResults, "w")
write.table(result, out, col.names = T, row.names = T, quote = F, sep = "\t")
close(out)

out <- file(outputSignResults, "w")
write.table(subset(result, padj < threshold), out, col.names = T, row.names = T, quote = F, sep = "\t")
close(out)
#--------------------------#
