
# commandArgs()[1] = HTSeq count table
# commandArgs()[2] = prefix of output files
# commandArgs()[3] = num. of replicates
# commandArgs()[4] = threshold

#----- library -----#
library(DESeq)
#-------------------#


#----- input -----#
args <- commandArgs(trailingOnly = TRUE)
inputFile <- args[1]
outputPrefix <- args[2]
NUM <- as.numeric(args[3])
threshold <- as.numeric(args[4])
data <- read.table(inputFile, header = T, row.names = 1)
outputTestResults <- paste(outputPrefix, ".results", sep = "")
outputSignResults <- paste(outputPrefix, ".significant", sep = "")
#-----------------#


#----- calc size factors -----#
cds <- newCountDataSet(data, rep(c("A", "B", "C", "D"), each = NUM))
cds <- estimateSizeFactors(cds)
#-----------------------------#


#----- estimate dispersions and perform statistical tests -----#
cds <- estimateDispersions(cds, method="blind", sharingMode="fit-only", fitType="local")
f.model <- fitNbinomGLMs(cds, count ~ condition)
r.model <- fitNbinomGLMs(cds, count ~ 1)
p <- nbinomGLMTest(f.model, r.model)
result <- data.frame(id = rownames(data), pval = p, qval = p.adjust(p, method = "BH"))
#--------------------------------------------------------------#


#----- output results -----#
out <- file(outputTestResults, "w")
write.table(result, out, col.names = T, row.names = F, quote = F, sep = "\t")
close(out)

out <- file(outputSignResults, "w")
write.table(subset(result, qval < threshold), out, col.names = T, row.names = F, quote = F, sep = "\t")
close(out)
#--------------------------#
