
# commandArgs()[1] = HTSeq count table
# commandArgs()[2] = prefix of output files
# commandArgs()[3] = num. of replicates
# commandArgs()[4] = threshold

#----- library -----#
library(edgeR)
#-------------------#


#----- input -----#
args <- commandArgs(trailingOnly = TRUE)
inputFile <- args[1]
outputPrefix <- args[2]
NUM <- as.numeric(args[3])
threshold <- as.numeric(args[4])
data <- read.table(inputFile, header = T, row.names = 1)
outputTestResults <- paste(outputPrefix, ".results", sep = "")
outputSignResults <- paste(outputPrefix, ".significant", sep = "")
#-----------------#


#----- normalization and dispersion  -----#
group <- factor(rep(c("A", "B", "C", "D"), each = NUM))
d <- DGEList(counts = data, group = group)
d <- calcNormFactors(d)
if(NUM == 1){
       	d <- estimateGLMCommonDisp(d, method = "deviance", robust = T, subset = NULL)
}else{
	d <- estimateCommonDisp(d)
	d <- estimateTagwiseDisp(d)
}
#-----------------------------------------#


#----- statistical test -----#
design <- model.matrix(~ group)
fit <- glmFit(d, design)
lrt <- glmLRT(fit, coef = c(2, 3, 4))
result <- topTags(lrt, n = nrow(data))
#----------------------------#


#----- output results -----#
out <- file(outputTestResults, "w")
write.table(result$table, out, col.names = T, row.names = T, quote = F, sep = "\t")
close(out)

out <- file(outputSignResults, "w")
write.table(subset(result$table, FDR < threshold), out, col.names = T, row.names = T, quote = F, sep = "\t")
close(out)
#--------------------------#
