
# commandArgs()[1] = HTSeq count table
# commandArgs()[2] = prefix of output files
# commandArgs()[3] = num. of replicates
# commandArgs()[4] = threshold

#----- library -----#
library(multiDE)
#-------------------#


#----- input -----#
args <- commandArgs(trailingOnly = TRUE)
inputFile <- args[1]
outputPrefix <- args[2]
NUM <- as.numeric(args[3])
threshold <- as.numeric(args[4])
data <- as.matrix(read.table(inputFile, header = T, row.names = 1))
outputTestResults <- paste(outputPrefix, ".results", sep = "")
outputSignResults <- paste(outputPrefix, ".significant", sep = "")
#-----------------#


#----- normalization and estimate dispersion -----#
M <- normalization(data, method = "median")
M <- dispersion(M, condition = rep(c("A", "B", "C", "D"), each = NUM), matched = TRUE, method = "edgeR")
#-------------------------------------------------#


#----- perform statistical tests -----#
DE <- multiDE(M)
result <- data.frame(id = rownames(data), pval = DE$p.value, qval = p.adjust(DE$p.value, method = "BH"))
#-------------------------------------#


#----- output results -----#
out <- file(outputTestResults, "w")
write.table(result, out, col.names = T, row.names = F, quote = F, sep = "\t")
close(out)

out <- file(outputSignResults, "w")
write.table(subset(result, qval < threshold), out, col.names = T, row.names = F, quote = F, sep = "\t")
close(out)
#--------------------------#
