
# commandArgs()[1]  = sorted_fpkm_mean_variance.
# commandArgs()[2]  = sorted_reference_transcriptome.
# commandArgs()[3]  = expression status for DEGs clusters.
# commandArgs()[4]  = list of fold-change for DEGs.
# commandArgs()[5]  = output file prefix.
# commandArgs()[6]  = number of replicates.
# commandArgs()[7]  = library sizes.                                (* 10^6)


#----- library -----#
library(MASS)
#-------------------#


#----- input -----#
args <- commandArgs(trailingOnly = TRUE)
inputFile <- args[1]
transcriptFile <- args[2]
testFile <- args[3]
foldChangeFile <- args[4]
outputFile <- paste(args[5], ".count", sep = "")
data <- read.table(inputFile, header = T, row.names = 1)
transcript <- read.table(transcriptFile, header = T, row.names = 1)
test <- read.table(testFile, header = T, row.names = 1)
foldChange <- read.table(foldChangeFile, header = F, row.names = 1)
message(paste("input file\t\t: ", inputFile, sep = ""))
message(paste("transcript file\t\t: ", transcriptFile, sep = ""))
message(paste("test file\t\t: ", testFile, sep = ""))
message(paste("fold-change file\t: ", foldChangeFile, sep = ""))
message(paste("output file\t\t: ", outputFile, sep = ""))
#-----------------#


#----- parameters -----#
condition <- 4
conditionList <- c("A", "B", "C", "D")
rep <- as.integer(args[6])
library_size <- as.numeric(args[7])
DEGsFrac <- 5
message(paste("replicate\t\t: ", rep, sep = ""))
message(paste("library size\t\t: ", library_size * 10^6, sep = ""))
message(paste("DEGs fraction\t\t: ", 1 / DEGsFrac, sep = ""))
#----------------------#


#----- output header -----#
headerFile <- paste("gene_id", "cluster", "fold-change", "length", sep = '\t')
sampleList <- paste("_sample", 1:rep, sep = "")
for(i in 1 : rep){
      for(j in 1 : condition){
            sample <- paste("condition", conditionList[j], sampleList[i], sep = "")
      	    headerFile <- paste(headerFile, sample, sep = "\t")
	}
}

outF <- file(outputFile, "w")
write(headerFile, outF)
close(outF)
#-------------------------#

message(paste("assign cluster number.", sep = ""))
#----- make uniform random numbers for deciding cluster -----#
DEbool <- floor(runif(nrow(data), min = 1, max = DEGsFrac + 1)) %/% DEGsFrac
cluster <- floor(runif(nrow(data), min = 1, max = nrow(test) + 1)) * DEbool
#------------------------------------------------------------#

message(paste("assign fold-change.", sep = ""))
#----- make uniform random numbers for deciding cluster -----#
frand <- floor(runif(nrow(data), min = 1, max = ncol(foldChange) + 1))
#------------------------------------------------------------#


#----- make total count matrix for each sample -----#
total_count <- matrix(0, nrow = 1, ncol = condition * rep)
tolerance <- matrix(0, nrow = nrow(data) + 1, ncol = 1)
tolerance[1, 1] = library_size * 10^6
#---------------------------------------------------#

message(paste("check the specified library size.", sep = ""))
#----- check the specified library size  -----#
T <- matrix(0, nrow = nrow(data), ncol = 2)
for(i in 1 : nrow(data)){
      mean_1 <- data[i, 2]
      var_1 <- data[i, 3]
      if((mean_1 * transcript[i, 1] * library_size / 10^3) < (var_1 * (transcript[i, 1] * library_size / 10^3)^2)){
      	T[i, 1] <- mean_1
      	T[i, 2] <- var_1
      }else{ stop("Library size is incorrect.\n") }
}
#---------------------------------------------#

message(paste("count simulation from NB distribution.", sep = ""))
#----- simulate negative binomial distribution -----#
i <- 1
while(i <= nrow(data)){
	gene_name <- substr(row.names(transcript[i, ]), 6, nchar(row.names(transcript[i, ])))
	count <- data.frame(GENE_NAME = gene_name, CLUATER_NUM = cluster[i], FOLD_CHANGE = foldChange[1, frand[i]] * DEbool[i], LENGTH = transcript[i, 1], COUNT = matrix(0, nrow = 1, ncol = condition * rep))
	mean_1 <- T[i, 1] * transcript[i, 1] * library_size * 10^6 / 10^9
	var_1 <- T[i, 2] * (transcript[i, 1] * library_size * 10^6 / 10^9)^2
	s_1 <- mean_1^2 / (var_1 - mean_1)
	t <- 0
	
	if(cluster[i] != 0){
	      	for(j in 1 : condition){
		      	if(test[cluster[i], j] > 0){
				for(k in seq(j, condition * rep, by = condition)){
			      	      count[1, k + 4] <- rnbinom(1, size = s_1, mu = mean_1) * foldChange[1, frand[i]]
				      t <- t + count[1, k + 4]
				      total_count[1, k] <- total_count[1, k] + count[1, k + 4]
				}
			}else{
				for(k in seq(j, condition * rep, by = condition)){
			      	      count[1, k + 4] <- rnbinom(1, size = s_1, mu = mean_1)
				      t <- t + count[1, k + 4]
				      total_count[1, k] <- total_count[1, k] + count[1, k + 4]
				}
			}
		}
	}else{
		for(j in 1:condition){
			for(k in seq(j, condition * rep, by = condition)){
		      	      count[1, k + 4] <- rnbinom(1, size = s_1, mu = mean_1)
			      t <- t + count[1, k + 4]
			      total_count[1, k] <- total_count[1, k] + count[1, k + 4]
			}
		}
	}
	for(j in 1:(condition * rep)){ tolerance[i + 1, 1] <- tolerance[i + 1, 1] + total_count[1, j] }
	tolerance[i + 1, 1] <- abs((library_size * 10^6) - (tolerance[i + 1, 1] / (condition * rep)))
	if(tolerance[i + 1, 1] <= tolerance[i, 1]){
		if(t == 0){
		     count[1, 2] = 0
		     count[1, 3] = 0
		}
		outF <- file(outputFile, "a")
		write.table(count, outF, quote = F, col.names = F, row.names = F, append = T, sep = "\t")
		close(outF)
		i <- i + 1
	}else{
		break
	}
}
#---------------------------------------------------#
